@@ -11,8 +11,7 @@
  * information regarding copyright ownership.
  */
 
-#ifndef DNS_DBITERATOR_H
-#define DNS_DBITERATOR_H 1
+#pragma once
 
 /*****
 ***** Module Info
@@ -69,7 +68,7 @@ ISC_LANG_BEGINDECLS
 *****/
 
 typedef struct dns_dbiteratormethods {
-	void (*destroy)(dns_dbiterator_t **iteratorp);
+	void	     (*destroy)(dns_dbiterator_t **iteratorp);
 	isc_result_t (*first)(dns_dbiterator_t *iterator);
 	isc_result_t (*last)(dns_dbiterator_t *iterator);
 	isc_result_t (*seek)(dns_dbiterator_t *iterator,
@@ -289,5 +288,3 @@ dns_dbiterator_setcleanmode(dns_dbiterator_t *iterator, bool mode);
  */
 
 ISC_LANG_ENDDECLS
-
-#endif /* DNS_DBITERATOR_H */
