@@ -1,6 +1,8 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at https://mozilla.org/MPL/2.0/.
@@ -9,8 +11,7 @@
  * information regarding copyright ownership.
  */
 
-#ifndef DNS_VIEW_H
-#define DNS_VIEW_H 1
+#pragma once
 
 /*****
 ***** Module Info
@@ -73,6 +74,7 @@
 #include <dns/rdatastruct.h>
 #include <dns/rpz.h>
 #include <dns/rrl.h>
+#include <dns/transport.h>
 #include <dns/types.h>
 #include <dns/zt.h>
 
@@ -81,16 +83,16 @@ ISC_LANG_BEGINDECLS
 struct dns_view {
 	/* Unlocked. */
 	unsigned int	  magic;
-	isc_mem_t *	  mctx;
+	isc_mem_t	 *mctx;
 	dns_rdataclass_t  rdclass;
-	char *		  name;
-	dns_zt_t *	  zonetable;
-	dns_resolver_t *  resolver;
-	dns_adb_t *	  adb;
+	char		 *name;
+	dns_zt_t	 *zonetable;
+	dns_resolver_t	 *resolver;
+	dns_adb_t	 *adb;
 	dns_requestmgr_t *requestmgr;
-	dns_cache_t *	  cache;
-	dns_db_t *	  cachedb;
-	dns_db_t *	  hints;
+	dns_cache_t	 *cache;
+	dns_db_t	 *cachedb;
+	dns_db_t	 *hints;
 
 	/*
 	 * security roots and negative trust anchors.
@@ -101,7 +103,7 @@ struct dns_view {
 
 	isc_mutex_t  lock;
 	bool	     frozen;
-	isc_task_t * task;
+	isc_task_t  *task;
 	isc_event_t  resevent;
 	isc_event_t  adbevent;
 	isc_event_t  reqevent;
@@ -111,11 +113,12 @@ struct dns_view {
 	bool	     cacheshared;
 
 	/* Configurable data. */
-	dns_tsig_keyring_t *  statickeys;
-	dns_tsig_keyring_t *  dynamickeys;
-	dns_peerlist_t *      peers;
-	dns_order_t *	      order;
-	dns_fwdtable_t *      fwdtable;
+	dns_transport_list_t *transports;
+	dns_tsig_keyring_t   *statickeys;
+	dns_tsig_keyring_t   *dynamickeys;
+	dns_peerlist_t	     *peers;
+	dns_order_t	     *order;
+	dns_fwdtable_t	     *fwdtable;
 	bool		      recursion;
 	bool		      qminimization;
 	bool		      qmin_strict;
@@ -130,24 +133,26 @@ struct dns_view {
 	bool		      trust_anchor_telemetry;
 	bool		      root_key_sentinel;
 	dns_transfer_format_t transfer_format;
-	dns_acl_t *	      cacheacl;
-	dns_acl_t *	      cacheonacl;
-	dns_acl_t *	      queryacl;
-	dns_acl_t *	      queryonacl;
-	dns_acl_t *	      recursionacl;
-	dns_acl_t *	      recursiononacl;
-	dns_acl_t *	      sortlist;
-	dns_acl_t *	      notifyacl;
-	dns_acl_t *	      transferacl;
-	dns_acl_t *	      updateacl;
-	dns_acl_t *	      upfwdacl;
-	dns_acl_t *	      denyansweracl;
-	dns_acl_t *	      nocasecompress;
+	dns_acl_t	     *cacheacl;
+	dns_acl_t	     *cacheonacl;
+	dns_acl_t	     *queryacl;
+	dns_acl_t	     *queryonacl;
+	dns_acl_t	     *recursionacl;
+	dns_acl_t	     *recursiononacl;
+	dns_acl_t	     *sortlist;
+	dns_acl_t	     *notifyacl;
+	dns_acl_t	     *transferacl;
+	dns_acl_t	     *updateacl;
+	dns_acl_t	     *upfwdacl;
+	dns_acl_t	     *denyansweracl;
+	dns_acl_t	     *nocasecompress;
 	bool		      msgcompression;
-	dns_rbt_t *	      answeracl_exclude;
-	dns_rbt_t *	      denyanswernames;
-	dns_rbt_t *	      answernames_exclude;
-	dns_rrl_t *	      rrl;
+	dns_rbt_t	     *answeracl_exclude;
+	dns_rbt_t	     *denyanswernames;
+	dns_rbt_t	     *answernames_exclude;
+	dns_rrl_t	     *rrl;
+	dns_rbt_t	     *sfd;
+	isc_rwlock_t	      sfd_lock;
 	bool		      provideixfr;
 	bool		      requestnsid;
 	bool		      sendcookie;
@@ -157,16 +162,16 @@ struct dns_view {
 	dns_ttl_t	      minncachettl;
 	uint32_t	      nta_lifetime;
 	uint32_t	      nta_recheck;
-	char *		      nta_file;
+	char		     *nta_file;
 	dns_ttl_t	      prefetch_trigger;
 	dns_ttl_t	      prefetch_eligible;
 	in_port_t	      dstport;
-	dns_aclenv_t	      aclenv;
+	dns_aclenv_t	     *aclenv;
 	dns_rdatatype_t	      preferred_glue;
 	bool		      flush;
-	dns_namelist_t *      delonly;
+	dns_namelist_t	     *delonly;
 	bool		      rootdelonly;
-	dns_namelist_t *      rootexclude;
+	dns_namelist_t	     *rootexclude;
 	bool		      checknames;
 	uint16_t	      maxudp;
 	dns_ttl_t	      staleanswerttl;
@@ -176,16 +181,16 @@ struct dns_view {
 	uint32_t	  staleanswerclienttimeout;
 	uint16_t	  nocookieudp;
 	uint16_t	  padding;
-	dns_acl_t *	  pad_acl;
+	dns_acl_t	 *pad_acl;
 	unsigned int	  maxbits;
 	dns_dns64list_t	  dns64;
 	unsigned int	  dns64cnt;
-	dns_rpz_zones_t * rpzs;
+	dns_rpz_zones_t	 *rpzs;
 	dns_catz_zones_t *catzs;
 	dns_dlzdblist_t	  dlz_searched;
 	dns_dlzdblist_t	  dlz_unsearched;
 	uint32_t	  fail_ttl;
-	dns_badcache_t *  failcache;
+	dns_badcache_t	 *failcache;
 
 	/*
 	 * Configurable data for server use only,
@@ -218,29 +223,29 @@ struct dns_view {
 	 * XXX: This should be a pointer to an opaque type that
 	 * named implements.
 	 */
-	char *	 new_zone_dir;
-	char *	 new_zone_file;
-	char *	 new_zone_db;
-	void *	 new_zone_dbenv;
-	uint64_t new_zone_mapsize;
-	void *	 new_zone_config;
-	void (*cfg_destroy)(void **);
+	char	   *new_zone_dir;
+	char	   *new_zone_file;
+	char	   *new_zone_db;
+	void	   *new_zone_dbenv;
+	uint64_t    new_zone_mapsize;
+	void	   *new_zone_config;
+	void	    (*cfg_destroy)(void **);
 	isc_mutex_t new_zone_lock;
 
 	unsigned char secret[32]; /* Client secret */
 	unsigned int  v6bias;
 
-	dns_dtenv_t *	dtenv;	 /* Dnstap environment */
+	dns_dtenv_t    *dtenv;	 /* Dnstap environment */
 	dns_dtmsgtype_t dttypes; /* Dnstap message types
 				  * to log */
 
 	/* Registered module instances */
 	void *plugins;
-	void (*plugins_free)(isc_mem_t *, void **);
+	void  (*plugins_free)(isc_mem_t *, void **);
 
 	/* Hook table */
 	void *hooktable; /* ns_hooktable */
-	void (*hooktable_free)(isc_mem_t *, void **);
+	void  (*hooktable_free)(isc_mem_t *, void **);
 };
 
 #define DNS_VIEW_MAGIC	     ISC_MAGIC('V', 'i', 'e', 'w')
@@ -250,6 +255,19 @@ struct dns_view {
 #define DNS_VIEWATTR_ADBSHUTDOWN 0x02
 #define DNS_VIEWATTR_REQSHUTDOWN 0x04
 
+#ifdef HAVE_LMDB
+#define DNS_LMDB_COMMON_FLAGS (MDB_CREATE | MDB_NOSUBDIR | MDB_NOLOCK)
+#ifndef __OpenBSD__
+#define DNS_LMDB_FLAGS (DNS_LMDB_COMMON_FLAGS)
+#else /* __OpenBSD__ */
+/*
+ * OpenBSD does not have a unified buffer cache, which requires both reads and
+ * writes to be performed using mmap().
+ */
+#define DNS_LMDB_FLAGS (DNS_LMDB_COMMON_FLAGS | MDB_WRITEMAP)
+#endif /* __OpenBSD__ */
+#endif /* HAVE_LMDB */
+
 isc_result_t
 dns_view_create(isc_mem_t *mctx, dns_rdataclass_t rdclass, const char *name,
 		dns_view_t **viewp);
@@ -378,9 +396,9 @@ dns_view_createzonetable(dns_view_t *view);
 
 isc_result_t
 dns_view_createresolver(dns_view_t *view, isc_taskmgr_t *taskmgr,
-			unsigned int ntasks, unsigned int ndisp,
-			isc_socketmgr_t *socketmgr, isc_timermgr_t *timermgr,
-			unsigned int options, dns_dispatchmgr_t *dispatchmgr,
+			unsigned int ntasks, unsigned int ndisp, isc_nm_t *nm,
+			isc_timermgr_t *timermgr, unsigned int options,
+			dns_dispatchmgr_t *dispatchmgr,
 			dns_dispatch_t *dispatchv4, dns_dispatch_t *dispatchv6);
 /*%<
  * Create a resolver and address database for the view.
@@ -392,7 +410,7 @@ dns_view_createresolver(dns_view_t *view, isc_taskmgr_t *taskmgr,
  *\li	'view' does not have a resolver already.
  *
  *\li	The requirements of dns_resolver_create() apply to 'taskmgr',
- *	'ntasks', 'socketmgr', 'timermgr', 'options', 'dispatchv4', and
+ *	'ntasks', 'nm', 'timermgr', 'options', 'dispatchv4', and
  *	'dispatchv6'.
  *
  * Returns:
@@ -440,6 +458,9 @@ dns_view_sethints(dns_view_t *view, dns_db_t *hints);
  * \li    	The hints database of 'view' is 'hints'.
  */
 
+void
+dns_view_settransports(dns_view_t *view, dns_transport_list_t *list);
+
 void
 dns_view_setkeyring(dns_view_t *view, dns_tsig_keyring_t *ring);
 void
@@ -803,6 +824,10 @@ dns_view_asyncload(dns_view_t *view, bool newonly, dns_zt_allloaded_t callback,
  *\li	'view' is valid.
  */
 
+isc_result_t
+dns_view_gettransport(dns_view_t *view, const dns_transport_type_t type,
+		      const dns_name_t *name, dns_transport_t **transportp);
+
 isc_result_t
 dns_view_gettsig(dns_view_t *view, const dns_name_t *keyname,
 		 dns_tsigkey_t **keyp);
@@ -930,7 +955,7 @@ dns_view_flushname(dns_view_t *view, const dns_name_t *name);
  *	other returns are failures.
  */
 
-isc_result_t
+void
 dns_view_adddelegationonly(dns_view_t *view, const dns_name_t *name);
 /*%<
  * Add the given name to the delegation only table.
@@ -944,7 +969,7 @@ dns_view_adddelegationonly(dns_view_t *view, const dns_name_t *name);
  *\li	#ISC_R_NOMEMORY
  */
 
-isc_result_t
+void
 dns_view_excludedelegationonly(dns_view_t *view, const dns_name_t *name);
 /*%<
  * Add the given name to be excluded from the root-delegation-only.
@@ -995,7 +1020,7 @@ dns_view_getrootdelonly(dns_view_t *view);
 isc_result_t
 dns_view_freezezones(dns_view_t *view, bool freeze);
 /*%<
- * Freeze/thaw updates to master zones.
+ * Freeze/thaw updates to primary zones.
  *
  * Requires:
  * \li	'view' is valid.
@@ -1195,7 +1220,7 @@ dns_view_ntacovers(dns_view_t *view, isc_stdtime_t now, const dns_name_t *name,
 
 void
 dns_view_untrust(dns_view_t *view, const dns_name_t *keyname,
-		 dns_rdata_dnskey_t *dnskey);
+		 const dns_rdata_dnskey_t *dnskey);
 /*%<
  * Remove keys that match 'keyname' and 'dnskey' from the views trust
  * anchors.
@@ -1212,6 +1237,19 @@ dns_view_untrust(dns_view_t *view, const dns_name_t *keyname,
  * \li	'dnskey' is valid.
  */
 
+bool
+dns_view_istrusted(dns_view_t *view, const dns_name_t *keyname,
+		   const dns_rdata_dnskey_t *dnskey);
+/*%<
+ * Determine if the key defined by 'keyname' and 'dnskey' is
+ * trusted by 'view'.
+ *
+ * Requires:
+ * \li	'view' is valid.
+ * \li	'keyname' is valid.
+ * \li	'dnskey' is valid.
+ */
+
 isc_result_t
 dns_view_setnewzones(dns_view_t *view, bool allow, void *cfgctx,
 		     void (*cfg_destroy)(void **), uint64_t mapsize);
@@ -1339,6 +1377,40 @@ dns_view_staleanswerenabled(dns_view_t *view);
  *\li	'view' to be valid.
  */
 
-ISC_LANG_ENDDECLS
+void
+dns_view_sfd_add(dns_view_t *view, const dns_name_t *name);
+/*%<
+ * Add 'name' to the synth-from-dnssec namespace tree for the
+ * view.  If the tree does not already exist create it.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ *\li	'name' to be valid.
+ */
 
-#endif /* DNS_VIEW_H */
+void
+dns_view_sfd_del(dns_view_t *view, const dns_name_t *name);
+/*%<
+ * Delete 'name' to the synth-from-dnssec namespace tree for
+ * the view when the count of previous adds and deletes becomes
+ * zero.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ *\li	'name' to be valid.
+ */
+
+void
+dns_view_sfd_find(dns_view_t *view, const dns_name_t *name,
+		  dns_name_t *foundname);
+/*%<
+ * Find the enclosing name to the synth-from-dnssec namespace tree for 'name'
+ * in the specified view.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ *\li	'name' to be valid.
+ *\li	'foundname' to be valid with a buffer sufficient to hold the name.
+ */
+
+ISC_LANG_ENDDECLS
